<?php
class BP_Maps_Map_Marker {

	var $ID;
	var $user_id;
	var $type;
	var $Lat;
	var $Lng;
	var $title;
	var $address;
	var $content;
	
	var $privacy;
	var $date_created;
	var $date_updated;
	
	function bp_maps_map_marker($id=null) {
		global $wpdb, $bp;
		
		if ( $id ) {
			$this->ID = $id;
		}

		$this->populate( $this->ID );
	}
	
	function populate() {
		global $bp;
		global $wpdb;
		
		$query = $wpdb->prepare( "SELECT * FROM {$bp->maps->table_name_markers} mk WHERE id={$this->ID}");
		
		if ( $row = $wpdb->get_row( $query ) ) {
			$this->user_id = apply_filters( 'bp_get_marker_type', $row->user_id );
			$this->type = apply_filters( 'bp_get_marker_type', $row->type );
			$this->Lat = apply_filters( 'bp_get_marker_lat',  $row->lat );
			$this->Lng = apply_filters( 'bp_get_marker_lng',  $row->lng );
			$this->title = apply_filters( 'bp_get_marker_title',  $row->title );
			$this->address = apply_filters( 'bp_get_marker_address',  $row->address );
			$this->content = apply_filters( 'bp_maps_get_marker_content',  $row->content,$this);
			$this->privacy = apply_filters( 'bp_get_marker_privacy',  $row->privacy );
			$this->date_created = apply_filters( 'bp_get_marker_date_created',  $row->date_created );
			$this->date_updated = apply_filters( 'bp_get_marker_date_updated',  $row->date_updated );
		}


	}
	
	function get_markers($markers_ids=false) {

		if (empty($markers_ids)) return false;
	
		if (!is_array($markers_ids)) {
			$markers_arr_ids[]=$markers_ids;
			$markers_ids = $markers_arr_ids;
		}else {
			$markers_ids = array_unique($markers_ids);
		}


		foreach ($markers_ids as $marker_id)
			$markers[]=new BP_Maps_Map_Marker($marker_id);
			
		return $markers;
	}
	
	function randomize_location($lat,$lng,$km=0.15) {
			
		$brg = rand(0,360);
		
		//Convert the starting point latitude 'lat1' (in the range -90 to 90) to radians.
		$lat1 = $lat * pi()/180;
		//Convert the starting point lnggitude 'lng1' (in the range -180 to 180) to radians.
		$lng1 = $lng * pi()/180;
		//Convert the bearing 'brg' (in the range 0 to 360) to radians.
		$brg = $brg * pi()/180;

		$radiusEarth = 6372.7976;
		//Convert distance to the distance in radians.
		$dist = $km/$radiusEarth;

		//Calculate the destination coordinates.
		$lat2 = asin(sin($lat1)*cos($dist) + cos($lat1)*sin($dist)*cos($brg));
		$lng2 = $lng1 + atan2(sin($brg)*sin($dist)*cos($lat1), cos($dist)-sin($lat1)*sin($lat2));
		
		//Calculate the final bearing and back bearing.
		$dLng = $lng1-$lng2;
		$y = sin($dLng) * cos($lat1);
		$x = cos($lat2)*sin($lat1) - sin($lat2)*cos($lat1)*cos($dLng);
		$d=atan2($y, $x);
		$finalBrg = $d+pi();
		$backBrg=$d+2*pi();
		//Convert lat2, lng2, finalBrg and backBrg to degrees
		$lat2 = $lat2 * 180/pi();
		$lng2 = $lng2 * 180/pi();
		$finalBrg = $finalBrg * 180/pi();
		$backBrg = $backBrg * 180/pi();


		// If lng2 is outside the range -180 to 180, add or subtract 360 to bring it back into that range.
		// If finalBrg or backBrg is outside the range 0 to 360, add or subtract 360 to bring them back into that range. 

		return array($lat2,$lng2);
	}
	
	function save() {
		global $wpdb, $bp;

		if (!$this->user_id)
				$this->user_id=$bp->loggedin_user->id;

		$this->user_id = apply_filters( 'maps_map_user_id_before_save', $this->user_id, $this->ID );
		$this->type = apply_filters( 'maps_map_type_before_save', $this->type, $this->ID );
		$this->Lat = apply_filters( 'maps_map_Lat_before_save', $this->Lat, $this->ID );
		$this->Lng = apply_filters( 'maps_map_Lng_before_save', $this->Lng, $this->ID );
		$this->title = apply_filters( 'maps_map_title_before_save', $this->title, $this->ID );
		$this->address = apply_filters( 'maps_map_address_before_save', $this->address, $this->ID );
 		$this->content = apply_filters( 'maps_map_content_before_save', $this->content, $this->ID );
		
		$this->privacy = apply_filters( 'maps_map_privacy_before_save', (array)$this->privacy, $this->ID );
		$this->date_created = apply_filters( 'maps_map_date_created_before_save', time() );

		do_action( 'maps_maps_marker_before_save', $this );
		


		if ( $this->ID ) {
			$sql = $wpdb->prepare(
				"UPDATE `{$bp->maps->table_name_markers}` SET
					`user_id` = %d,
					`type` = %s,
					`lat` = %s,
					`lng` = %s,
					`title` = %s,
					`address` = %s,
					`content` = %s,
					`privacy` = %s,
					`date_updated` = %s
				WHERE
					id = %d
				",
					$this->user_id,
					$this->type,
					$this->Lat,
					$this->Lng,
					$this->title,
					$this->address,
					$this->content,
					$this->privacy,
					$this->date_created,
					$this->ID
			);
		} else {
			$sql = $wpdb->prepare(
				"INSERT INTO `{$bp->maps->table_name_markers}` (
					`user_id`,
					`type`,
					`lat`,
					`lng`,
					`title`,
					`address`,
					`content`,
					`privacy`,
					`date_created`,
					`date_updated`
				) VALUES (
					%d, %s, %s, %s, %s, %s, %s, %s, %s, %s
				)",
					$this->user_id,
					$this->type,
					$this->Lat,
					$this->Lng,
					$this->title,
					$this->address,
					$this->content,
					$this->privacy,
					$this->date_created,
					$this->date_updated
			);
		}

		if ( false === $wpdb->query($sql) )
			return false;

		if ( !$this->ID ) {
			$this->ID = $wpdb->insert_id;
		}

		do_action( 'maps_maps_marker_after_save', $this );

		return $this->ID;
	}
	
	function delete() {
		global $wpdb, $bp;

		do_action( 'bp_maps_delete_map', $this );

		// Finally remove the map entry from the DB
		if ( !$wpdb->query( $wpdb->prepare( "DELETE FROM {$bp->maps->table_name_markers} WHERE id = %d", $this->ID ) ) )
			return false;

		return true;
	}

}




/*****************************************************************************
 * Maps Template Class
 **/

class Bp_Map {
	var $ID; //unique ID | needed for the CSS|JS

	var $display='both'; //static, dynamic or both 
	var $size;
	var $width;
	var $height;
	
	var $editable; //markers editables
	var $enable_desc; // title & description allowed (when $editable=true)
	
	var $showmarkers=false;
	var $markers_max; //max markers
	var $markers=array();
	
	var $Lat;
	var $Lng;
	
	var $zoom;
	var $mapTypeControlOptions;
	var $navigationControlOptions;
	var $mapTypeId;
	
	var $image;
	var $type;

	function bp_map($markers_ids,$args=false) {
		global $bp;
		
		if ($bp->maps->options['map']['options']['center'])
			$default_center = explode(",",$bp->maps->options['map']['options']['center']);

		$defaults = array(
			'display'	=>$bp->maps->options['map']['options']['display'],
			'size'		=>$bp->maps->options['map']['options']['size'],
			'showmarkers'	=> true,
			'enable_desc'	=>$bp->maps->options['map']['options']['enable_desc'],
			'markers_max' => $bp->maps->options['map']['options']['markers_max'],
			'lat'			=>$default_center[0],
			'lng'			=>$default_center[1],
			'zoom'			=>$bp->maps->options['map']['options']['zoom'],
			'mapTypeControlOptions'	=>$bp->maps->options['map']['options']['mapTypeControlOptions'],
			'navigationControlOptions'=>$bp->maps->options['map']['options']['navigationControlOptions'],
			'mapTypeId'		=>$bp->maps->options['map']['options']['mapTypeId'],
			
		);
		
		$r = wp_parse_args( $args, $defaults );

		extract( $r, EXTR_SKIP );

		//CUSTOM SIZE OR NOT
		if ($width && $height) {
			$this->width=$width;
			$this->height=$height;
		}else {
			//default size key
			$key = $bp->maps->options['map']['options']['size'];
			$this->width = $bp->maps->options['map']['sizes'][$key]['width'];
			$this->height = $bp->maps->options['map']['sizes'][$key]['height'];
		}
		
		$this->display=$display;
		
		//MAX MARKERS
		$this->markers_max=$markers_max;
		
		//LAT/LNG
		$this->Lat=$lat;
		$this->Lng=$lng;
		$this->zoom=$zoom;

		$this->mapTypeControlOptions=$mapTypeControlOptions;
		$this->navigationControlOptions=$navigationControlOptions;
		$this->mapTypeId=$mapTypeId;
		
		//EDITABLE OR NOT
		if ($editable) {
			$this->editable=$editable;
			$this->enable_desc=$enable_desc;
			$this->type=$type;
			
		}

		//MARKERS
		if (!is_array($markers_ids))
			$markers_ids[]=$markers_ids;

		$markers_ids=apply_filters('bp_maps2_markers_ids',$markers_ids);

		$this->markers = BP_Maps_Map_Marker::get_markers($markers_ids);
		
		//one single marker
		if ((count($this->markers))==1) {
			if (!bp_maps_marker_user_can_view_exact($this->markers[0])) {
				$point=BP_Maps_Map_Marker::randomize_location($this->markers[0]->Lat,$this->markers[0]->Lng);
			}else {
				$point = array($this->markers[0]->Lat,$this->markers[0]->Lng);
			}
			$this->Lat=$point[0];
			$this->Lng=$point[1];
		}

		//Zoom, lat & lng are redifned in get_static_image

		if ($this->display!='dynamic')
			$this->image=self::get_static_image();

		
		//five alphanumeric chars, like the JS FN
		$key = uniqid();
		$key = rand();
		$key = substr($key, 0, 3);
		$this->ID="map_".$key;
	}
	
	function get_static_image() {
		require_once ( BP_MAPS_PLUGIN_DIR . '/_inc/php/static-maps/Maps.php' );
		global $bp;
		
		///MARKERS
		if ($this->markers) {
			foreach ($this->markers as $marker) {
				if ((!$marker->Lat) || (!$marker->Lng)) continue;
				
				//PRIVACY | HIDE
				if (!bp_maps_marker_user_can_view($marker))	continue;
				
				//PRIVACY | APPROX POINT
				if (!bp_maps_marker_user_can_view_exact($marker)) {
					$point=BP_Maps_Map_Marker::randomize_location($marker->Lat,$marker->Lng);
					$marker->Lat = $point[0];
					$marker->Lng = $point[1];
				}
				
				
				$coord = new Google_Maps_Coordinate($marker->Lat, $marker->Lng);
				$marker = new Google_Maps_Marker($coord);
				//$marker->setColor('green');
				/*
				$marker->content = "test";
				
				if ($marker->content) {
					$bubble = new Google_Maps_Infowindow($marker->content);
					$bubble->setMarker($marker);
					$map->addInfowindow($bubble);
				}
				*/
				
				$markers[]=$marker;
			}
		}
		///MARKERS
		
		///MAP
		//init
		$map=Google_Maps::create('static');

		//center
		$center=new Google_Maps_Coordinate($this->Lat,$this->Lng);
		$map->setCenter($center);
		$map->setSize($this->width.'x'.$this->height);
		$map->setKey($bp->maps->options['api_key']);

		$map->setZoom($this->zoom);
		
	
		if ($markers)
			$map->setMarkers($markers);
		///MAP
		
		
		//MAP CENTER
		if (count($this->markers)>1) { //several markers
			$map->zoomToFit();
		}
		
		$this->zoom=$map->getZoom();

		$this->Lat = $map->getCenter()->getLat();
		$this->Lng = $map->getCenter()->getLon();

		return $map;
	}
}



?>