<?php
require ( BP_MAPS_PLUGIN_DIR . '/bp-maps-classes.php' );
require ( BP_MAPS_PLUGIN_DIR . '/bp-maps-templatetags.php' );
require ( BP_MAPS_PLUGIN_DIR . '/bp-maps-ajax.php' );
require ( BP_MAPS_PLUGIN_DIR . '/bp-maps-filters.php' );
require ( BP_MAPS_PLUGIN_DIR . '/bp-maps-admin.php' );

function bp_maps_install() {
	global $wpdb, $bp;

	if ( !empty($wpdb->charset) )
		$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
		
	$sql[] = "CREATE TABLE {$bp->maps->table_name_markers} (
		`id` bigint(20) unsigned NOT NULL auto_increment,
		`user_id` bigint(20) unsigned NOT NULL default '0',
		`type` varchar(20) NOT NULL,
		`lat` varchar(64) NOT NULL,
		`lng` varchar(64) NOT NULL,
		`title` text NOT NULL,
		`address` text NOT NULL,
		`content` text NOT NULL,
		`privacy` text NOT NULL,
		`date_created` datetime NOT NULL,
		`date_updated` datetime NOT NULL,
		PRIMARY KEY  (`id`)
	 	   ) {$charset_collate};";


	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
	dbDelta($sql);

	do_action( 'bp_maps_install' );

	update_site_option( 'bp-maps-db-version', BP_GROUPS_DB_VERSION );
}

function maps_check_installed() {	
	global $wpdb, $bp;

	//Need to check db tables exist, activate hook no-worky in mu-plugins folder.
	if ( get_site_option('bp-maps-db-version') < BP_MAPS_DB_VERSION )
		bp_maps_install();
}
add_action( 'admin_menu', 'maps_check_installed' );

function bp_maps_is_setup() {
	
	//
	return true;

	$maps_options = get_site_option( 'bp_maps_options');
	
	if ($maps_options['blog_id']) {
		return true;
	}
}


function maps_setup_globals() {
	global $bp, $wpdb;

	/* For internal identification */
	$bp->maps->id = 'maps';
	
	/* Register this in the active components array */
	//$bp->maps->format_notification_function = 'maps_format_notifications';
	$bp->maps->slug = BP_MAPS_SLUG;

	/* Register this in the active components array */
	$bp->active_components[$bp->maps->slug] = $bp->maps->id;

	
	$options_defaults['map']['options'] =(array(
			'markers_max'=>5,
			'enable_desc'=>true,
			'display'=>'both',
			'zoom'=>15,
			'mapTypeControlOptions'=>'DROPDOWN_MENU',
			'navigationControlOptions'=>'SMALL','ANDROID',
			'mapTypeId'=>'ROADMAP'
		));
	
	$options_defaults['map']['sizes'] = array (
							'SMALL' => array('width' => '300', 'height' => '225'),
							'MEDIUM' => array('width' => '400', 'height' => '300'),
							'LARGE' => array('width' => '640', 'height' => '480') 
							);
							
	//PRIVACY
	//action=>privacy level
	//by order of importance
	$options_defaults['marker']['privacy']['marker_view'] = false;
	$options_defaults['marker']['privacy']['marker_view_approx'] = 0;
	$options_defaults['marker']['privacy']['marker_view_full'] = 1;

	
	$custom_options = get_site_option( 'bp_maps_options');

	$bp->maps->options = wp_parse_args($custom_options, $options_defaults );

	$bp->maps->table_name_markers = $wpdb->base_prefix . 'bp_maps_markers';
	

	
	//level=>name of the filtering check function
	//by order of importance !
	$bp->maps->privacy['levels'] = array (
		'visitor'=>'bp_maps_check_is_visitor',
		'member'=>'bp_maps_check_is_member',
		'friend'=>'bp_maps_check_is_friend'
	);	

	//CAPABILITIES
	$capabilities	= array(
		'maps_view_markers' => array(	
			'name'	=> __('View Markers','bp-maps'),
			'group'	=> 'markers',
			'default'	=>true,
		),
		'maps_view_markers_approximate' => array(	
			'name'	=> __('View Approximate locations only','bp-maps'),
			'group'	=> 'markers'
		),
		'maps_create_markers' => array(	
			'name'	=> __('Create Markers','bp-maps'),
			'group'	=> 'my-markers',
			'default'	=>true,
		),
		'maps_marker_creation_visitors_hide' => array(	'name'	=> __('Visitors','maps'),
			'group'	=> 'my-markers-hide',
			'default'	=>true,
		),
		'maps_marker_creation_members_hide' => array(	'name'	=> __('Members','maps'),
			'group'	=> 'my-markers-hide',
			'default'	=>true,
		),
		'maps_marker_creation_friends_hide' => array(	'name'	=> __('Friends','maps'),
			'group'	=> 'my-markers-hide',
			'default'	=>true,
		),
		'maps_marker_creation_visitors_show_approx' => array(	'name'	=> __('Visitors','maps'),
			'group'	=> 'my-markers-show-approx',
			'default'	=>true,
		),
		'maps_marker_creation_members_show_approx' => array(	'name'	=> __('Members','maps'),
			'group'	=> 'my-markers-show-approx',
			'default'	=>true,
		),
		'maps_marker_creation_friends_show_approx' => array(	'name'	=> __('Friends','maps'),
			'group'	=> 'my-markers-show-approx',
			'default'	=>true,
		),

	);
	$bp->maps->capabilities = apply_filters( 'maps_capabilities',$capabilities);
	
	//activity
	apply_filters( 'bp_activity_mini_activity_types', array(
		'marker_created') );

	do_action('bp_maps_init');
	
}
add_action( 'plugins_loaded', 'maps_setup_globals', 5 );
add_action( 'admin_menu', 'maps_setup_globals', 2 );


// Adds admin menu to WP Dashboard > BuddyPress	
function bp_maps_add_admin_menu() {
	global $wpdb, $bp;
	

	if ( !is_site_admin() )
		return false;

	/* Add the administration tab under the "Site Admin" tab for site administrators */
	add_submenu_page( 'bp-general-settings', __('Maps','bp-maps'),__('Maps','bp-maps'), 'manage_options', 'bp-maps-setup', "bp_maps_admin" );
}
add_action('admin_menu', 'bp_maps_add_admin_menu');


///HEAD|START///
	/**
	* Scripts for non-admin screens
	*
	*/
	
	function bp_maps_print_head() {
		$js[]="\n<script type='text/javascript'>";
		$js[]="<!--BP-MAPS|START-->";
		$js[]="var geocoder;";
		$js[]="<!--BP-MAPS|END-->";
		$js[]="</script>\n";
		echo implode("\n",$js);
	}
	
	function bp_maps_print_scripts() {
		wp_enqueue_script('bp-maps-unobstrusive',BP_MAPS_PLUGIN_URL . '/_inc/js/unobstrusive-gmaps.js', array('jquery'),BP_MAPS_VERSION);
		wp_enqueue_script('jquery.json', BP_MAPS_PLUGIN_URL . '/_inc/js/jquery.json-2.2.min.js',array('jquery'), '2.2' );
		wp_enqueue_script('bp-maps',BP_MAPS_PLUGIN_URL . '/_inc/js/bp-maps.js', array('jquery'),BP_MAPS_VERSION);
	}

	/**
	* Stylesheets for non-admin pages
	*
	*/
	function bp_maps_print_styles() {
		wp_enqueue_style( 'bp-maps-screen', apply_filters('bp_maps_enqueue_url',get_stylesheet_directory_uri() . '/maps/style.css'));
	}
///HEAD|END///


function array_flatten($array, $preserve_keys = false)
{
  if (!$preserve_keys) {
    // ensure keys are numeric values to avoid overwritting when array_merge gets called
    $array = array_values($array);
  }

  $flattened_array = array();
  foreach ($array as $k => $v) {
    if (is_array($v)) {
      $flattened_array = array_merge($flattened_array, call_user_func(__FUNCTION__, $v, $preserve_keys));
    } elseif ($preserve_keys) {
      $flattened_array[$k] = $v;
    } else {
      $flattened_array[] = $v;
    }
  }
  return $flattened_array;
}

//CAPABILITIES

function bp_maps_get_roles_names() {
	$roles_names['default']=__('(no role)', 'bp-maps');
	$roles_names=apply_filters('bp_maps_roles_names',$roles_names);
	return $roles_names;
}


function bp_maps_get_user_role($user_id=false) {
	global $bp;
	
	if (!$user_id) {
		global $bp;
		$user_id = $bp->loggedin_user->id;
	}

	//if (!$user_id) return 'visitor';
	
	$user = new WP_User( $user_id );

	if ( !empty( $user->roles ) && is_array( $user->roles ) ) {
		foreach ( $user->roles as $role )
			return $role;
	}

}

function bp_map_user_can_default($action) {
	global $bp;
	$capabilities = $bp->maps->capabilities;
	
	if (
	($capabilities[$action]['group']=='maps') || //visitors
	(($capabilities[$action]['group']=='my-maps') && ($bp->loggedin_user->domain)) //logged user
	)
		return true; 
}

function bp_map_user_can($action) {
	return true;
	//always OK when is site admin
	//if (current_user_can('level_10')) return true;

	$role = bp_maps_get_roles_names();
	
	$default_capabilities = get_option('maps_r&c_default');
	
	if (!$default_capabilities) return bp_map_user_can_default($action);

	if ((!$role) || (!bp_maps_plugin_is_active('buddypress-maps/bp-maps-roles-and-capabilities.php'))) {
		return $default_capabilities[$action];
	}else {
		return current_user_can($action);
	}
}

//PRIVACY CHECK FUNCTIONS
function bp_maps_check_is_friend($marker_author_id=false) {
	//TO FIX
	return true;
	global $bp;
	if (BP_Friends_Friendship::check_is_friend( $bp->loggedin_user->id, $marker_author_id )=='is_friend')
		return true;
		
	return false;
}
function bp_maps_check_is_member() {
	if (is_user_logged_in())
		return true;
		
	return false;
}
function bp_maps_check_is_visitor() {
	if (!is_user_logged_in())
		return true;
		
	return false;
}

//PRIVACY FUNCTIONS
function bp_maps_get_user_privacy_level($user_id=false) {
	global $bp;
	
	if (!$user_id) $user_id = $bp->loggedin_user->id;
	
	if (!$user_id) return false;
	
	$index=-1;
	foreach ($bp->maps->privacy['levels'] as $name => $fn) {
		if ($fn()) $index++;
	}
	return $index;
}

function bp_maps_marker_user_can_view ($marker) {

	//if (current_user_can('level_10')) return true;

	//if ($marker->user_id=$bp->loggedin_user->id) return true;
	
	$needed_level = $marker->privacy['marker_view'];
	
	if (!$needed_level) return true;
	
	if ($user_level >= bp_maps_get_user_privacy_level()) return true;
	
	return false;
	
}

function bp_maps_marker_user_can_view_approx ($marker) {

	//if (current_user_can('level_10')) return true;

	//if ($marker->user_id=$bp->loggedin_user->id) return true;
	
	$needed_level = $marker->privacy['marker_view_approx'];
	
	if (!$needed_level) return true;
	
	if ($user_level >= bp_maps_get_user_privacy_level()) return true;
	
	return false;
	
}

function bp_maps_marker_user_can_view_exact($marker) {

	//if (current_user_can('level_10')) return true;

	//if ($marker->user_id=$bp->loggedin_user->id) return true;
	
	$needed_level = $marker->privacy['marker_view_full'];
	
	if (!$needed_level) return true;
	
	if ($user_level >= bp_maps_get_user_privacy_level()) return true;
	
	return false;
	
}

function bp_maps_head_init() {
	//WILL NOT RUN MORE THAN ONCE.
	add_action('wp_head','bp_maps_print_head');
	add_action('wp_print_scripts','bp_maps_print_scripts');
	add_action('wp_print_styles','bp_maps_print_styles');
}



?>